@@ -11,6 +11,7 @@ use secrecy::ExposeSecret;
 use secrecy::SecretString;
 use serde::{de, ser};
 
+use crate::location::Location;
 use crate::output::Output;
 use crate::v1;
 
@@ -47,7 +48,7 @@ impl Api {
     }
 
     pub(crate) async fn is_unauthorized(&self) -> bool {
-        self.get_all_states()
+        self.get_all_clusters()
             .await
             .err()
             .and_then(|e| e.downcast::<reqwest::Error>().ok())
@@ -59,7 +60,29 @@ impl Api {
         self.post("/states", state).await
     }
 
-    pub(crate) async fn delete_state(&self, name: v1::StateName) -> ApiResult<v1::State> {
+    pub(crate) async fn create_volume(
+        &self,
+        state_name: v1::StateName,
+        volume: v1::CreateVolumeDto,
+    ) -> ApiResult<v1::Volume> {
+        let path = format!("/states/{state_name}/volumes", state_name = state_name);
+        self.post(path, volume).await
+    }
+
+    pub(crate) async fn delete_volume(
+        &self,
+        state: v1::StateName,
+        volume: v1::VolumeName,
+    ) -> ApiResult<v1::Volume> {
+        let path = format!(
+            "/states/{name}/volumes/{volume}",
+            name = state,
+            volume = volume
+        );
+        self.del(path).await
+    }
+
+    pub(crate) async fn delete_state(&self, name: v1::StateName) -> ApiResult<()> {
         let path = format!("/states/{name}", name = name);
         self.del(path).await
     }
@@ -73,17 +96,27 @@ impl Api {
         self.get("/states").await
     }
 
-    pub(crate) async fn list_clusters(&self) -> ApiResult<Vec<v1::Cluster>> {
+    pub(crate) async fn get_all_clusters(&self) -> ApiResult<Vec<v1::Cluster>> {
         self.get("/clusters").await
     }
 
-    pub(crate) async fn register_cluster(&self, name: &v1::ClusterName) -> ApiResult<v1::Cluster> {
+    pub(crate) async fn register_cluster(
+        &self,
+        name: &v1::ClusterName,
+        provider: v1::Provider,
+        locations: &[Location],
+    ) -> ApiResult<v1::Cluster> {
         let name = name.clone();
-        let body = v1::CreateClusterDto { name };
+        let locations = locations.into();
+        let body = v1::CreateClusterDto {
+            name,
+            provider,
+            locations,
+        };
         self.post("/clusters", body).await
     }
 
-    pub(crate) async fn unregister_cluster(&self, name: v1::ClusterName) -> ApiResult<v1::Cluster> {
+    pub(crate) async fn unregister_cluster(&self, name: v1::ClusterName) -> ApiResult<()> {
         let path = format!("/clusters/{name}", name = name);
         self.del(path).await
     }
@@ -94,7 +127,7 @@ impl Api {
         region: v1::AwsRegion,
     ) -> ApiResult<v1::StateLocationAws> {
         let path = format!("/states/{name}/locations/aws", name = name);
-        let body = v1::CreateStateLocationAws { region };
+        let body = v1::CreateStateLocationAwsDto { region };
         self.post(path, body).await
     }
 
@@ -104,7 +137,7 @@ impl Api {
         region: v1::AzureRegion,
     ) -> ApiResult<v1::StateLocationAzure> {
         let path = format!("/states/{name}/locations/azure", name = name);
-        let body = v1::CreateStateLocationAzure { region };
+        let body = v1::CreateStateLocationAzureDto { region };
         self.post(path, body).await
     }
 
@@ -150,13 +183,8 @@ impl Api {
     pub(crate) async fn unset_owner(
         &self,
         state: impl AsRef<v1::StateName>,
-        cluster: impl AsRef<v1::ClusterName>,
     ) -> ApiResult<v1::State> {
-        let path = format!(
-            "/states/{state}/owner/{cluster}",
-            state = state.as_ref(),
-            cluster = cluster.as_ref(),
-        );
+        let path = format!("/states/{state}/owner", state = state.as_ref());
         self.del(path).await
     }
 
@@ -165,7 +193,7 @@ impl Api {
         cluster: impl AsRef<v1::ClusterName>,
     ) -> ApiResult<v1::ClusterToken> {
         let path = format!("/clusters/{}/token", cluster.as_ref());
-        self.post(path, "").await
+        self.post::<_, _, (), _>(path, None).await
     }
 
     fn url(&self, path: impl fmt::Display) -> String {
@@ -217,18 +245,21 @@ impl Api {
             .inspect(|output| self.inspect(output))
     }
 
-    async fn post<P, T, U>(&self, path: P, body: T) -> ApiResult<U>
+    async fn post<P, B, T, U>(&self, path: P, body: B) -> ApiResult<U>
     where
         P: fmt::Display,
+        B: Into<Option<T>>,
         T: ser::Serialize,
         U: de::DeserializeOwned + ser::Serialize + fmt::Debug,
     {
+        let body = body.into();
         let url = self.url(path);
         self.client()?
             .post(url)
             .optionally_bearer_auth(self.token.as_ref())
             .inspect()
-            .json(&body)
+            .optionally_json(body.as_ref())
+            // .json(&body)
             .send()
             .await?
             .error_for_status2()
@@ -264,6 +295,9 @@ impl Api {
 
 trait Optionally {
     fn optionally_bearer_auth(self, token: Option<&SecretString>) -> Self;
+    fn optionally_json<T>(self, body: Option<&T>) -> Self
+    where
+        T: ser::Serialize;
 
     fn optionally<T, F>(self, option: Option<T>, f: F) -> Self
     where
@@ -283,6 +317,13 @@ impl Optionally for reqwest::RequestBuilder {
         let token = token.map(ExposeSecret::expose_secret);
         self.optionally(token, Self::bearer_auth)
     }
+
+    fn optionally_json<T>(self, body: Option<&T>) -> Self
+    where
+        T: ser::Serialize,
+    {
+        self.optionally(body, Self::json)
+    }
 }
 
 trait Inspector {
